#include <math.h>
#include <stdlib.h>
#include <formatio.h>
#include <cvirte.h>     /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <visa.h>
#include "rsupl.h"
#include "rsupl_fftp.h"

static int configHdl, panelHdl;
static ViSession instrSession;
static ViStatus status;
static ViReal64 data[2048], resolution;
static ViInt32 pts;

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)   /* Needed if linking in external compiler; harmless otherwise */
        return -1;  /* out of memory */
    if ((configHdl = LoadPanel (0, "rsupl_fftp.uir", CONFIG)) < 0)
        return -1;
    if ((panelHdl = LoadPanel (0, "rsupl_fftp.uir", PANEL)) < 0)
        return -1;
    DisplayPanel (panelHdl);
    InstallPopup (configHdl);
    RunUserInterface ();
    return 0;
}

void errorQuit(char* func, ViStatus status) {
    static char errMsg[512];
    static char label[256];
    if (func != 0) {
        Fmt (label, "%s<%s ERROR", func);
    } else {
        Fmt (label, "%s<ERROR");
    }
    rsupl_error_message (instrSession, status, errMsg);
    MessagePopup (label, errMsg);
    exit (0);
}

int CVICALLBACK okCall (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    char strBuf[256];
    int reset;
    ViUInt32 retCnt;
    switch (event)
        {
        case EVENT_COMMIT:
            // initialize and reset to a known state
            GetCtrlVal (configHdl, CONFIG_INSTRDESCR, strBuf);
            GetCtrlVal (configHdl, CONFIG_RESET, &reset);
            if ((status = rsupl_init (strBuf, VI_TRUE, (ViBoolean)reset, &instrSession)) < 0) {
                errorQuit("rsupl_init", status);
            }

            // config trigger
            if ((status = rsupl_confInstrTrig (instrSession, VI_FALSE)) < 0) {
                errorQuit("rsupl_confInstrTrig", status);
            }
            
            // config universal analog generator
            if ((status = rsupl_confGenAnStp (instrSession, 3, VI_FALSE, 0, 10.0, 1000.0,
                                        1.0, 0, 0)) < 0) {
                errorQuit("rsupl_confGenAnStp", status);
            }
            
            // config DFD signal output
            if ((status = rsupl_confGenFuncDFD (instrSession, VI_FALSE, 0.0, 0, 1.0, 0,
                                                VI_FALSE, 10.0, 1.0, 1, VI_FALSE, 0.0, 15,
                                                0, VI_FALSE, VI_FALSE, "*.veq")) < 0) {
                errorQuit("rsupl_confGenFuncDFD", status);
            }
            
            // config 22kHz analog analyzer
            if ((status = rsupl_confAnalyzerAnalStp (instrSession, 600.0, 0, VI_TRUE,
													 VI_FALSE, 1, 1, VI_FALSE,
													 VI_FALSE, 0, 0, 0, 0, VI_FALSE)) < 0) {
                errorQuit("rsupl_confAnalyzerAnalStp", status);
            }
            
            // config FFT
            if ((status = rsupl_confAnalyzerFuncFFT (instrSession, VI_FALSE, 0, 10000.0,
                                                     1, 0, 3, 1, 6, 1.0, 0, 4, 0, 0,
                                                     10.0, 12.0)) < 0) {
                errorQuit("rsupl_confAnalyzerFuncFFT", status);
            }
            
            // enhance measurement speed
            // switch off display & cursors
            if ((status = rsupl_confInstrDisplay (instrSession,VI_FALSE, VI_FALSE)) < 0) {
                errorQuit("rsupl_confInstrDisplay", status);
            }
            // switch off sense 2 & 3
            if ((status = rsupl_confInstrSense (instrSession,VI_TRUE, VI_FALSE, VI_FALSE)) < 0) {
                errorQuit("rsupl_confInstrDisplay", status);
            }
            
            // LOW-LEVEL PROGRAMMING:
            // query FFT resolution for spectrum graph
            if ((status = viWrite (instrSession, (ViByte *)"\n:CALC:TRAN:FREQ:RES?\n", 22, &retCnt)) < 0) {
                errorQuit("viWrite", status);
            }
            if ((status = viRead (instrSession, (ViByte *)strBuf, 50, &retCnt)) < 0) {
                errorQuit("viRead", status);
            }
            Fmt (&resolution, "%f[b8]<%s", strBuf);
            // config display for spectrum
            if ((status = viWrite (instrSession, (ViByte *)"\n:DISP:TRAC1:FEED 'SENS1:DATA1'\n", 32, &retCnt)) < 0) {
                errorQuit("viWrite", status);
            }
            
            // remove configuration dialog
            RemovePopup (1);
            
            // run measurement (see timerCallback())
            SetCtrlAttribute (panelHdl, PANEL_TIMER, ATTR_ENABLED, 1);
            break;
        }
    return 0;
}

int CVICALLBACK timerCall (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32 i;
    ViPReal64 p2data;

    switch (event)
        {
        case EVENT_TIMER_TICK:
            // send trigger
            if ((status = rsupl_sendTrig (instrSession)) < 0) {
                errorQuit("rsupl_sendTrig", status);
            }
            
            // read data
            if ((status = rsupl_dataReadBlock (instrSession, 11, data, &pts)) < 0) {
                errorQuit("rsupl_dataReadBlock", status);
            }
            
            for (i = 0, p2data = data; i < pts; i++, p2data++) {
                *p2data = 20.0 * log10(*p2data);
            }
            
            // display data
            DeleteGraphPlot (panelHdl, PANEL_GRAPH, -1, VAL_DELAYED_DRAW);
			PlotWaveform (panelHdl, PANEL_GRAPH, data, pts, VAL_DOUBLE, 1.0, 0.0,
						  0.0, resolution, VAL_THIN_LINE, VAL_EMPTY_SQUARE,
						  VAL_SOLID, 1, VAL_GREEN);
            break;
        }
    return 0;
}

int CVICALLBACK quitCall (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            // stop measurement
            SetCtrlAttribute (panelHdl, PANEL_TIMER, ATTR_ENABLED, 0);
            QuitUserInterface (0);
            break;
        }
    return 0;
}

